
#============================================================================
# Tangent-V specific Indexing parameter
#
# Kenny Davila
# - Created:  January 2018
#
#============================================================================

from AM_CommonTools.configuration.configuration import Configuration

class IndexParameters:

    IndexEntry_MinCosineSimilarity = 0.707107  # 0.0 (0.0 deg.), 0.707107 (45 deg.)
    IndexEntry_ScaleCosineSimilarity = False
    IndexEntry_MinSizeRatio = 0.0

    Graph_KNN = 2
    Graph_TopClasses = 10
    Graph_MinScale = 0.333
    Graph_MaxScale = 3.0
    Graph_MaxEdgeDistance = 2.0

    Pairs_MinConfidence = 0.8
    Pairs_SmallExpressions = 0

    Match_MaxPathLength = 4
    Match_MaxSpatialScore = 3.0
    Match_MaxDisconnectedSpatialScore = 3.0
    Match_EarlyDeletion = True
    Match_MinGroupAreaOverlap = 0.5
    Match_Wildcards = False

    Metric_ProductBased = 0                  # original
    Metric_MaximumLineOfSightSimilarity = 1  # MLS
    Metric_MinBased = 2                      # Use min confidence for matching labels
    Metric_MinBasedMLS = 3                   # MLS - using min confidence for weighted recall

    Top_Layer1_candidates = 100
    Layer1_ReturnRankings = False

    # TODO: this could belong to a retrieval parameters
    Layer2_JoinDisconnectedMatches = False
    Layer3_PrecisionScore = 0

    SimilarityMetric = 0

    @staticmethod
    def ReadFromConfiguration(config):
        assert isinstance(config, Configuration)

        if config.contains("ENTRY_MIN_COS_SIMILARITY"):
            IndexParameters.IndexEntry_MinCosineSimilarity = config.get_float("ENTRY_MIN_COS_SIMILARITY")
        if config.contains("ENTRY_SCALE_COS_SIMILARITY"):
            IndexParameters.IndexEntry_ScaleCosineSimilarity = config.get_bool("ENTRY_SCALE_COS_SIMILARITY")
        if config.contains("ENTRY_MIN_SIZE_RATIO"):
            IndexParameters.IndexEntry_MinSizeRatio = config.get_float("ENTRY_MIN_SIZE_RATIO")

        if config.contains("GRAPH_KNN"):
            IndexParameters.Graph_KNN = config.get_int("GRAPH_KNN")
        if config.contains("GRAPH_TOP_CLASSES"):
            IndexParameters.Graph_TopClasses = config.get_int("GRAPH_TOP_CLASSES")
        if config.contains("GRAPH_MIN_SCALE"):
            IndexParameters.Graph_MinScale = config.get("GRAPH_MIN_SCALE")
        if config.contains("GRAPH_MAX_SCALE"):
            IndexParameters.Graph_MaxScale = config.get("GRAPH_MAX_SCALE")
        if config.contains("GRAPH_MAX_EDGE_DISTANCE"):
            IndexParameters.Graph_MaxEdgeDistance = config.get_float("GRAPH_MAX_EDGE_DISTANCE")

        if config.contains("PAIRS_MIN_CONFIDENCE"):
            IndexParameters.Pairs_MinConfidence = config.get_float("PAIRS_MIN_CONFIDENCE")
        if config.contains("PAIRS_SMALL_EXPRESSION"):
            IndexParameters.Pairs_SmallExpressions = config.get_int("PAIRS_SMALL_EXPRESSION")

        if config.contains("MATCH_MAX_PATH_LENGTH"):
            IndexParameters.Match_MaxPathLength = config.get_int("MATCH_MAX_PATH_LENGTH")
        if config.contains("MATCH_MAX_SPATIAL_SCORE"):
            IndexParameters.Match_MaxSpatialScore = config.get_float("MATCH_MAX_SPATIAL_SCORE")
        if config.contains("MATCH_MAX_DISCONNECTED_SPATIAL_SCORE"):
            IndexParameters.Match_MaxDisconnectedSpatialScore = config.get_float("MATCH_MAX_DISCONNECTED_SPATIAL_SCORE")
        if config.contains("MATCH_MIN_GROUP_AREA_OVERLAP"):
            IndexParameters.Match_MinGroupAreaOverlap = config.get_float("MATCH_MIN_GROUP_AREA_OVERLAP")
        if config.contains("MATCH_EARLY_DELETION"):
            IndexParameters.Match_EarlyDeletion = config.get_bool("MATCH_EARLY_DELETION")
        if config.contains("MATCH_WILDCARDS"):
            IndexParameters.Match_Wildcards = config.get_bool("MATCH_WILDCARDS")

        if config.contains("SIMILARITY_METRIC"):
            IndexParameters.SimilarityMetric = config.get_int("SIMILARITY_METRIC")

        if config.contains("TOP_LAYER1_CANDIDATES"):
            IndexParameters.Top_Layer1_candidates = config.get_int("TOP_LAYER1_CANDIDATES")

        if config.contains("LAYER1_RETURN_INITIAL_RANKINGS"):
            IndexParameters.Layer1_ReturnRankings = config.get_bool("LAYER1_RETURN_INITIAL_RANKINGS")

        if config.contains("LAYER2_JOIN_DISCONNECTED_MATCHES"):
            IndexParameters.Layer2_JoinDisconnectedMatches = config.get_bool("LAYER2_JOIN_DISCONNECTED_MATCHES")

        if config.contains("LAYER3_PRECISION_SCORE"):
            IndexParameters.Layer3_PrecisionScore = config.get_int("LAYER3_PRECISION_SCORE")
